from abc import ABC, abstractmethod
from dataclasses import dataclass
from typing import Self

from tiko.terminal import Terminal


class ModuleDependencyError(Exception):
    pass


class InstallConfirmationError(Exception):
    pass


@dataclass
class Module(ABC):
    terminal: Terminal
    dependencies: list[Self] | None = None

    @abstractmethod
    def install(self) -> None:
        pass

    def check_if_dependencies_processed(self, processed_module_classes: list[Self]) -> None:
        if self.dependencies is None:
            return
        for dependency in self.dependencies:
            if not dependency not in processed_module_classes:
                raise ModuleDependencyError(f'{type(self)} expects {dependency} to be processed before it.')



from tiko.module import Module, InstallConfirmationError


class RustModule(Module):
    def check_if_installed(self) -> bool:
        return self.terminal.check_if_command_exists('cargo')

    def install(self) -> None:
        if not self.check_if_installed():
            self.terminal.run_command('curl https://sh.rustup.rs -sSf | sh -s -- -y')
            if not self.check_if_installed():
                raise InstallConfirmationError





import shutil
import tomllib
from dataclasses import dataclass
from pathlib import Path
from typing import Self

import logging
from tiko.logging import set_up_default_logger
from tiko.module_processing import process_list
from tiko.terminal import Terminal


@dataclass
class Installer:
    terminal: Terminal

    @classmethod
    def new(cls) -> Self:
        set_up_default_logger()
        instance = cls(Terminal.new())
        return instance

    def install(self) -> None:
        tiko_configuration_path = Path('tiko_configuration.toml')
        if tiko_configuration_path.exists():
            with tiko_configuration_path.open('rb') as tiko_configuration_file:
                tiko_configuration_dictionary = tomllib.load(tiko_configuration_file)
        process_list(['rust', 'nu', 'zellij', 'bottom', 'dua'], self.terminal)


if __name__ == '__main__':
    logger = logging.getLogger('tiko')
    logger.setLevel(logging.DEBUG)
    installer = Installer.new()
    installer.install()



from pexpect import spawn

import logging
from dataclasses import dataclass
from typing import Self


logger = logging.getLogger(__name__)


class TerminalError(Exception):
    pass


prompt_prefix = '<>prompt_prefix<>'


@dataclass
class Terminal:
    process: spawn

    @classmethod
    def new(cls) -> Self:
        process = spawn('/bin/bash', encoding='utf-8')
        instance = cls(process=process)
        process.sendline(f'export PS1="{prompt_prefix}> "')
        process.expect(prompt_prefix, timeout=None)
        process.expect(prompt_prefix, timeout=None)
        return instance

    def run_command(self, command: str) -> str:
        logger.debug(f'Sending command: `{command}`')
        self.process.sendline(command)
        self.process.expect(prompt_prefix, timeout=None)
        output = self.log_and_return_before()
        return output

    def log_and_return_before(self) -> str:
        output = self.process.before
        output = output.replace('\r\n', '\n')
        for output_line in output.splitlines():
            logger.debug('Pexpect output start.')
            logger.info(output_line)
            logger.debug('Pexpect output end.')
        return output

    def check_if_command_exists(self, command: str) -> bool:
        output = self.run_command(f'which {command}')
        output_lines = output.splitlines()
        return len(output_lines) > 1

    def install_cargo_crate(self, crate_name: str) -> None:
        locked_install_output = self.run_command(f'cargo install --locked {crate_name}')
        if 'error: failed to compile' in locked_install_output:
            unlocked_install_output = self.run_command(f'cargo install {crate_name}')
            if 'error: failed to compile' in unlocked_install_output:
                raise TerminalError(f'Failed to install cargo crate {crate_name}.')



from tiko.module import Module
from tiko.modules.bottom_module import BottomModule
from tiko.modules.dua_module import DuaModule
from tiko.modules.nu_module import NuModule
from tiko.modules.rust_module import RustModule
from tiko.modules.zellij_module import ZellijModule
from tiko.terminal import Terminal

module_name_to_class_mapping = {
    'rust': RustModule,
    'nu': NuModule,
    'zellij': ZellijModule,
    'bottom': BottomModule,
    'dua': DuaModule,
}


def process_list(module_name_list: list[str], terminal: Terminal) -> None:
    processed_module_classes: list[Module] = []
    for module_name in module_name_list:
        module_class = module_name_to_class_mapping[module_name]
        module = module_class(terminal)
        module.check_if_dependencies_processed(processed_module_classes=processed_module_classes)
        module.install()
        processed_module_classes.append(module_class)



import logging
import sys

from tiko.logging import set_up_default_logger, create_default_formatter


def pytest_configure(config):
    set_up_default_logger()
    formatter = create_default_formatter()
    handler = logging.StreamHandler(sys.stdout)
    handler.setLevel(logging.DEBUG)
    handler.setFormatter(formatter)
    logger = logging.getLogger('tiko')
    logger.addHandler(handler)
    logger.setLevel(logging.DEBUG)



from tiko.module import Module, InstallConfirmationError
from tiko.modules.rust_module import RustModule


class BottomModule(Module):
    dependencies = [RustModule]

    def check_if_installed(self) -> bool:
        return self.terminal.check_if_command_exists('btm')

    def install(self) -> None:
        if not self.check_if_installed():
            self.terminal.install_cargo_crate('bottom')
            if not self.check_if_installed():
                raise InstallConfirmationError



from tiko.module import Module, InstallConfirmationError
from tiko.modules.rust_module import RustModule
from tiko.terminal import Terminal


class NuModule(Module):
    dependencies = [RustModule]

    def check_if_installed(self) -> bool:
        return self.terminal.check_if_command_exists('nu')

    def install(self) -> None:
        if not self.check_if_installed():
            self.terminal.install_cargo_crate('nu')
            self.terminal.install_cargo_crate('nu_plugin_polars')
            # TODO: Run `plugin add ~/.cargo/bin/nu_plugin_polars` inside a nu instance.
            if not self.check_if_installed():
                raise InstallConfirmationError


if __name__ == '__main__':
    module = NuModule(Terminal.new())
    module.install()




import datetime
import logging
import sys
from pathlib import Path

logger_initialized = False


def create_default_formatter() -> logging.Formatter:
    formatter = logging.Formatter('tiko [{asctime} {levelname} {name}] {message}', style='{')
    return formatter


def set_up_default_logger():
    global logger_initialized
    if not logger_initialized:
        formatter = create_default_formatter()
        handler = logging.FileHandler(Path('tiko_install.log'))
        handler.setLevel(logging.DEBUG)
        handler.setFormatter(formatter)
        logger = logging.getLogger('tiko')
        logger.addHandler(handler)
        logger.setLevel(logging.INFO)
        logger.propagate = False
        sys.excepthook = excepthook
        logger_initialized = True


def excepthook(exc_type, exc_value, exc_traceback):
    logger = logging.getLogger('tiko')
    logger.critical(f'Uncaught exception at {datetime.datetime.now()}:')
    logger.handlers[0].flush()
    sys.__excepthook__(exc_type, exc_value, exc_traceback)
